"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codegen_1 = require("@azure-tools/codegen");
const doc_comments_1 = require("./doc-comments");
const expression_1 = require("./expression");
const field_1 = require("./field");
const type_1 = require("./type");
const linq_1 = require("@azure-tools/linq");
function sortByNamePartialFirst(a, b) {
    if (a.isPartial !== b.isPartial) {
        return a.isPartial ? -1 : 1;
    }
    return codegen_1.sortByName(a, b);
}
exports.sortByNamePartialFirst = sortByNamePartialFirst;
class Class extends type_1.Type {
    constructor(namespace, name, parent, objectIntializer) {
        super(namespace, name);
        this.parent = parent;
        this.classOrStruct = 'class';
        this.isStatic = false;
        this.fields = new Array();
        this.partial = true;
        this.apply(objectIntializer);
        namespace.addClass(this);
        if (!this.description.trim()) {
            this.description = '';
        }
    }
    hasMethodWithSameDeclaration(method) {
        for (const eachMethod of this.methods) {
            if (method.interfaceDeclaration === eachMethod.interfaceDeclaration) {
                return true;
            }
        }
        return false;
    }
    get headerComment() {
        return codegen_1.comment(doc_comments_1.xmlize('summary', this.description), codegen_1.docCommentPrefix) || '';
    }
    get signature() {
        const colon = (this.parent || linq_1.length(this.interfaces) > 0) ? ' : ' : '';
        const comma = (this.parent && linq_1.length(this.interfaces) > 0) ? ', ' : '';
        const extendsClass = this.parent ? this.parent.fullName : '';
        const implementsInterfaces = this.interfaces.map(v => v.declaration).join(',\n    ');
        const partial = this.partial ? 'partial ' : '';
        const stat = this.isStatic ? 'static ' : '';
        return `
${this.headerComment}
${this.attributeDeclaration}${this.accessModifier} ${stat}${partial}${this.classOrStruct} ${this.name}${colon}${extendsClass}${comma}
    ${implementsInterfaces}
`.trim();
    }
    get definition() {
        const fields = this.fields.sort(codegen_1.sortByName).map(m => m.declaration).join(codegen_1.EOL + codegen_1.EOL);
        const methods = this.methods.sort(sortByNamePartialFirst).map(m => m.implementation).join(codegen_1.EOL + codegen_1.EOL);
        const properties = this.properties.sort(codegen_1.sortByName).map(m => m.declaration).join(codegen_1.EOL + codegen_1.EOL);
        return `
${this.signature}
{
${codegen_1.indent(fields, 1)}

${codegen_1.indent(properties, 1)}

${codegen_1.indent(methods, 1)}
}
`.trim();
    }
    add(item) {
        if (item instanceof field_1.Field) {
            this.fields.push(item);
            return item;
        }
        return super.add(item);
    }
    addField(field) {
        this.fields.push(field);
        return field;
    }
    get declaration() {
        return `
${this.fullName}
`.trim();
    }
    new(...parameters) {
        return expression_1.toExpression(`new ${this.name}(${parameters.joinWith(each => expression_1.valueOf(each))})`);
    }
    property(name) {
        return this.properties.find(p => p.name === name) || codegen_1.fail(`No property ${name}`);
    }
    $(name) {
        return this.properties.find(p => p.name === name) ||
            this.methods.find(p => p.name === name) ||
            this.fields.find(p => p.name === name) || codegen_1.fail(`No Property/Method/Field ${name}`);
    }
}
exports.Class = Class;
//# sourceMappingURL=class.js.map