"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
const codegen_1 = require("@azure-tools/codegen");
const access_modifier_1 = require("./access-modifier");
const doc_comments_1 = require("./doc-comments");
const expression_1 = require("./expression");
const parameter_1 = require("./parameter");
const statement_1 = require("./statements/statement");
const Void = {
    declaration: 'void'
};
class Method extends statement_1.Statements {
    constructor(name, returnType = Void, objectIntializer) {
        super();
        this.name = name;
        this.returnType = returnType;
        this.parameters = new Array();
        this['new'] = access_modifier_1.Modifier.None;
        this.access = access_modifier_1.Access.Public;
        this['static'] = access_modifier_1.Modifier.None;
        this.virtual = access_modifier_1.Modifier.None;
        this.sealed = access_modifier_1.Modifier.None;
        this.override = access_modifier_1.Modifier.None;
        this.abstract = access_modifier_1.Modifier.None;
        this.extern = access_modifier_1.Modifier.None;
        this.async = access_modifier_1.Modifier.None;
        this.isPartial = false;
        this.description = '';
        this.returnsDescription = '';
        this.apply(objectIntializer);
        // easy access to allow statements in the initalizer.
        if (this.body && !expression_1.isAnExpression(this.body)) {
            this.add(this.body);
        }
        if (!this.description.trim()) {
            this.description = '';
        }
        if (!this.returnsDescription.trim()) {
            this.returnsDescription = '';
        }
    }
    addParameter(parameter) {
        this.parameters.push(parameter);
        return parameter;
    }
    get summaryDocumentation() {
        return codegen_1.docComment(doc_comments_1.summary(this.description));
    }
    get parameterDocumentation() {
        return codegen_1.docComment(this.parameters.joinWith(p => p.comment, codegen_1.EOL));
    }
    get returnsDocumentation() {
        if (this.returnType.declaration !== 'void') {
            return codegen_1.docComment(doc_comments_1.xmlize('returns', this.returnsDescription));
        }
        return '';
    }
    get declaration() {
        const parameterDeclaration = this.parameters.joinWith(p => p.declaration, codegen_1.CommaChar);
        const doccomment = `${this.summaryDocumentation}
${this.parameterDocumentation}
${this.returnsDocumentation}`.replace(/\s*\n/g, '\n').replace(/\n+/g, '\n').trim();
        return `${doccomment}
${this.new}${this.access} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.async} ${this.returnType.declaration} ${this.name}(${parameterDeclaration})
`.slim();
    }
    get interfaceDeclaration() {
        const parameterDeclaration = this.parameters.joinWith(p => p.declaration, codegen_1.CommaChar);
        const doccomment = `${this.summaryDocumentation}
${this.parameterDocumentation}
${this.returnsDocumentation}`.replace(/\s*\n/g, '\n').replace(/\n+/g, '\n').trim();
        return `${doccomment}
${this.returnType.declaration} ${this.name}(${parameterDeclaration});
`.slim();
    }
    get implementation() {
        if (expression_1.isAnExpression(this.body)) {
            return `
${this.declaration} => ${this.body.value};
`.trim();
        }
        return `
${this.declaration}
{
${codegen_1.indent(super.implementation)}
}`.trim();
    }
    invoke(...parameters) {
        return expression_1.toExpression(`${this.name}(${parameters.joinWith(expression_1.valueOf)})`);
    }
    addTo(parent) {
        parent.addMethod(this);
        return this;
    }
}
exports.Method = Method;
class PartialMethod extends Method {
    constructor(name, returnType = Void, objectIntializer) {
        super(name, returnType);
        this.isPartial = true;
        this.access = access_modifier_1.Access.Default;
        this.apply(objectIntializer);
    }
    get declaration() {
        const parameterDeclaration = this.parameters.joinWith(p => p.declaration, codegen_1.CommaChar);
        return `
${this.summaryDocumentation}
${this.parameterDocumentation}
${this.returnsDocumentation}
partial ${this.new}${this.access} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.async} ${this.returnType.declaration} ${this.name}(${parameterDeclaration})
`.slim();
    }
    get implementation() {
        return `${this.declaration};`.slim();
    }
}
exports.PartialMethod = PartialMethod;
class LambdaMethod extends Method {
    constructor(name, returnType = Void, expression, objectIntializer) {
        super(name, returnType);
        this.expression = expression;
        this.apply(objectIntializer);
    }
    get declaration() {
        const parameterDeclaration = this.parameters.joinWith(p => p.declaration, codegen_1.CommaChar);
        return `
${this.summaryDocumentation}
${this.parameterDocumentation}
${this.returnsDocumentation}
${this.new}${this.access} ${this.static} ${this.virtual} ${this.sealed} ${this.override} ${this.abstract} ${this.extern} ${this.async} ${this.returnType.declaration} ${this.name}(${parameterDeclaration}) => ${expression_1.valueOf(this.expression)}
`.slim();
    }
    get implementation() {
        return `${this.declaration};`.slim();
    }
}
exports.LambdaMethod = LambdaMethod;
class ImplicitCastOperator extends Method {
    constructor(targetType, sourceType, expression, objectIntializer) {
        super(`implicit operator ${targetType.declaration}`, targetType, { parameters: [new parameter_1.Parameter('source', sourceType)], static: access_modifier_1.Modifier.Static, });
        this.expression = expression;
    }
    get declaration() {
        const parameterDeclaration = this.parameters.joinWith(p => p.declaration, codegen_1.CommaChar);
        return `
${this.summaryDocumentation}
${this.parameterDocumentation}
${this.returnsDocumentation}
${this.new}${this.access} static ${this.override} ${this.abstract} implicit operator ${this.returnType.declaration}(${parameterDeclaration}) => ${expression_1.valueOf(this.expression)}
`.slim();
    }
    get implementation() {
        return `${this.declaration};`.slim();
    }
}
exports.ImplicitCastOperator = ImplicitCastOperator;
//# sourceMappingURL=method.js.map